// Naam         : BrainBoxCar 
// Omschrijving : Programma voor het testen van de verschillende onderdelen/functies van de auto
//                selecteer een mode met switch D22, 
//                activeer de mode met switch D8 
//                stop het programma met de RESET switch
//
//               MODE                                              | D11 | D13 | D30 | D17 |
//               1 - Buzzer + leds                                     0     0     0     1
//               2 - Alle sensoren mbv SERIAL MONITOR                  0     0     1     0
//               3 - DC motor test                                     0     0     1     1
//               4 - DC motor drive                                    0     1     0     0
//               5 - Servo motor                                       0     1     0     1
//               6 - Lichtzoeker met fototrans (A                      0     1     0     0
//               7 - Afstandmeter met IR-sensoren(D9/A2 & D10/A3)      0     1     1     1
//               8 - Lijnvolger (sensoren onder auto)   sensoren)      1     0     0     0
//               9 - bluetooth                                         1     0     0     1
//   
// 
// Auteur : F. Bremer
// Datum  : 6 -6 - 2017
// rev    : 2.1

#include "pitches.h"           // bestand met noten voor muziekstukje, Moet in zelfde map als hoofdprogramma staan
#include <Wire.h>              // I2C lib
#include <LiquidCrystal_I2C.h> // https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
#include <Servo.h>             // servo lib
#include <avr/io.h>            // om echte AVR C-code te kunnen schrijven 
#include <avr/interrupt.h>     // interrupts voor AVR-microcontroller

// ----- Declare Constants ----- 
#define F_CPU 16000000                                                                      // Xtal op 16MHz
#define USART_BAUDRATE 9600                                                                 // Baudrate op 9600bps - voor BT comms
#define BAUD_PRESCALE (((( F_CPU / 16) + ( USART_BAUDRATE / 2) ) / ( USART_BAUDRATE )) - 1) // Baudrate settings voor BT comms

// *** pinnummer naar pinnaam ***
#define ACT_SW   8            // activate test mode switch  
#define MODE_SW  22           // select mode switch

#define LGHT_SNR_LFT    A0   // light dependent resistor on left side of auto
#define LGHT_SNR_RGHT   A1   // light dependent resistor on right side of auto

#define DIST_ACT_LFT    9    // Infra red transmitter on left site of auto
#define DIST_SNR_LFT    A2   // Infra led sensor on left side of auto
#define DIST_ACT_RGHT   10   // Infra red transmitter on right site of auto
#define DIST_SNR_RGHT   A3   // Infra led sensor on right side of auto

#define SPD_MTR_LFT     6    // speed of left motor
#define DIR_MTR_LFT     14   // direction (forward / backward) of left motor
#define SPD_MTR_RGHT    5    // speed of right motor
#define DIR_MTR_RGHT    16   // direction (forward / backward) of right motor

#define LINE_SNR_LFT    4    // line follower sensor on left site of bottom auto
#define LINE_SNR_RGHT   12   // line follower sensor on lef site of bottom auto

#define SERVO_1   15         // servo 1
#define SERVO_2   23         // servo 2

#define LED_M0  17           // mode led 0 (lsb)
#define LED_M1  30           // mode led 1
#define LED_M2  13           // mode led 2
#define LED_M3  11           // mode led 3 (msb)

#define BUZZER 7
   
// ----- Declare Objects ----- 
// Set the LCD address to 0x27 for a 20 chars 2 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol 
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);   

Servo servo_1;
Servo servo_2;

// *** function prototypes ***
void music();
void serialmonitor();
void DC_motor_1();
void DC_motor_2();
void servo();
void lightfollower();
void distance();
void linefollower();
void bluetooth();

void DC_Motor_Stop();   // Stop both wheels
void DC_Motor_Forwards(byte spd_Lft, byte spd_Rght);    // direction Motor forwards (lOW), full speed = 255, stop = 0
void DC_Motor_Backwards(byte spd_Lft, byte spd_Rght);   // direction Motor backwards (HIGH), full speed = 255, stop = 0
void DC_Motor_Spinning_Left(byte spd_Lft, byte spd_Rght);    // direction Motor Left backwards (HIGH), righ forwards (LOW), full speed = 255, stop = 0
void DC_Motor_Spinning_Right(byte spd_Lft, byte spd_Rght);    // direction Motor Left forwards (LOW), righ backwards (HIGH), full speed = 255, stop = 0

// ----- Declare Global Variables -----
byte mode;

volatile byte state = LOW; 
volatile byte ReceiveX;
volatile byte ReceiveY;
volatile byte startcomms = 0;

// *** Initialisatie BrainboxCar ***
void setup()
{
  // define input/output 
  pinMode( ACT_SW, INPUT ); 
  pinMode( MODE_SW, INPUT );

  pinMode( DIST_ACT_LFT , OUTPUT );
  pinMode( DIST_ACT_RGHT , OUTPUT );
  
  pinMode( LINE_SNR_LFT, INPUT );
  pinMode( LINE_SNR_RGHT, INPUT );

  pinMode( SPD_MTR_LFT, OUTPUT ); 
  pinMode( SPD_MTR_RGHT, OUTPUT ); 
  pinMode( DIR_MTR_LFT , OUTPUT );
  pinMode( DIR_MTR_RGHT , OUTPUT );
  
  pinMode( LED_M0, OUTPUT );
  pinMode( LED_M1, OUTPUT );
  pinMode( LED_M2, OUTPUT );
  pinMode( LED_M3, OUTPUT );

  servo_1.attach(SERVO_1);
  servo_2.attach(SERVO_2);

  DC_Motor_Stop();   // zet beide DC motoren stil

  // *** display startup ***
  lcd.begin(16,2);      // Initialize the lcd for 16 chars 2 lines, turn on backlight
  
  lcd.setCursor(0,0);   // Start at 1e karakter of 1e line
  lcd.print("BOL4 Elektrotech");   // print text on the LCD
  delay(200);
  lcd.setCursor(4,1);       // set cursor at 1e karakter of 2e line 
  lcd.print("ROBOTICA");    // print text on the LCD
  delay(1500); 

  for(int i = 0; i < 3; i++)   // loop 3 times backlight on and off
  {
    lcd.backlight();          // backlight on
    delay(250);
    lcd.noBacklight();        // backlight off
    delay(250);
  }
  lcd.backlight();             //backlight on
  
  lcd.clear();
  lcd.print("BRAINBOXCAR     ");    // print text on the LCD              
}

void loop()
{  
  bool modeSwitch;
  bool activateSwitch;

  mode = 1;
  
  //  *** Keuzemenu ***
  // elke keuze activeert een ander subprogramma
  while ( !digitalRead(ACT_SW) )  // activateswitch not active
  {
    if ( digitalRead(MODE_SW) )   // modeswitch active 
    {
      mode++; 
      if (mode == 10)
      {
        mode = 1 ;
      }
    }   

    lcd.setCursor(0,1);
    lcd.print(mode);

    switch (mode)
    {
      case 1:
        lcd.print(":Music         ");        
        digitalWrite( LED_M0, HIGH );
        digitalWrite( LED_M1, LOW );
        digitalWrite( LED_M2, LOW );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 2:
        lcd.print(":SerialMonitor ");        
        digitalWrite( LED_M0, LOW );
        digitalWrite( LED_M1, HIGH );
        digitalWrite( LED_M2, LOW );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 3:
        lcd.print(":DC-motor 1    ");             
        digitalWrite( LED_M0, HIGH );
        digitalWrite( LED_M1, HIGH);
        digitalWrite( LED_M2, LOW );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 4:
        lcd.print(":DC-motor 2    ");          
        digitalWrite( LED_M0, LOW );
        digitalWrite( LED_M1, LOW);
        digitalWrite( LED_M2, HIGH );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 5:
        lcd.print(":Servo         ");      ;      
        digitalWrite( LED_M0, HIGH );
        digitalWrite( LED_M1, LOW);
        digitalWrite( LED_M2, HIGH );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 6:
        lcd.print(":Lightfollower ");            
        digitalWrite( LED_M0, LOW );
        digitalWrite( LED_M1, HIGH);
        digitalWrite( LED_M2, HIGH );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 7:
        lcd.print(":Obstacle avoid");                      
        digitalWrite( LED_M0, HIGH );
        digitalWrite( LED_M1, HIGH);
        digitalWrite( LED_M2, HIGH );
        digitalWrite( LED_M3, LOW );
        break;
        
      case 8:
        lcd.print(":Linefollower  ");                  
        digitalWrite( LED_M0, LOW );
        digitalWrite( LED_M1, LOW);
        digitalWrite( LED_M2, LOW );
        digitalWrite( LED_M3, HIGH );
        break;  
    
      case 9:     
        lcd.print(":Bluetooth     ");                      
        digitalWrite( LED_M0, HIGH );
        digitalWrite( LED_M1, LOW);
        digitalWrite( LED_M2, LOW );
        digitalWrite( LED_M3, HIGH );
    }    
    delay(300);
  }

  lcd.setCursor(2,1);
  lcd.print("Running       ");
   
  switch (mode)
  {
    case 1:
      music();
      break;

    case 2:
      serialmonitor();
      break;

    case 3:
      DC_motor_1();
      break; 

    case 4:
      DC_motor_2();
      break;         

    case 5:
      servo();
      break;         

    case 6:
      lightfollower();
      break;

    case 7:
      distance();
      break;

    case 8:
      linefollower();
      break;

    case 9:
      bluetooth();
      break;
  }
}

void DC_Motor_Stop()
{
  // stop both wheels
  digitalWrite(DIR_MTR_LFT, LOW);   
  analogWrite(SPD_MTR_LFT, 0); 
  digitalWrite(DIR_MTR_RGHT, LOW );
  analogWrite(SPD_MTR_RGHT, 0);
}

void DC_Motor_Forwards(byte spd_Lft, byte spd_Rght)
{
  // direction Motor forwards(lOW), full speed = 255, stop = 0
  digitalWrite(DIR_MTR_LFT, LOW);   
  analogWrite(SPD_MTR_LFT, spd_Lft); 
  digitalWrite(DIR_MTR_RGHT, LOW );
  analogWrite(SPD_MTR_RGHT, spd_Rght);
}

void DC_Motor_Backwards(byte spd_Lft, byte spd_Rght)
{
  // direction Motor backwards(HIGH), full speed = 255, stop = 0
  digitalWrite(DIR_MTR_LFT, HIGH);   
  analogWrite(SPD_MTR_LFT, 255 - spd_Lft); 
  digitalWrite(DIR_MTR_RGHT, HIGH );
  analogWrite(SPD_MTR_RGHT, 255 - spd_Rght);
}

void DC_Motor_Spinning_Left(byte spd_Lft, byte spd_Rght)    
{
  // direction Motor Left backwards (HIGH), righ forwards(LOW), full speed = 255, stop = 0
  digitalWrite(DIR_MTR_LFT, HIGH);   
  analogWrite(SPD_MTR_LFT, 255 - spd_Lft); 
  digitalWrite(DIR_MTR_RGHT, LOW );
  analogWrite(SPD_MTR_RGHT, spd_Rght);  
}

void DC_Motor_Spinning_Right(byte spd_Lft, byte spd_Rght) 
{
  // direction Motor Left forwards(LOW), righ backwards(HIGH), full speed = 255, stop = 0
  digitalWrite(DIR_MTR_LFT, LOW);   
  analogWrite(SPD_MTR_LFT, spd_Lft); 
  digitalWrite(DIR_MTR_RGHT, HIGH );
  analogWrite(SPD_MTR_RGHT, 255 - spd_Rght);  
}
 
void music()
{
  // korte melodie Mario Bros
  int melody[] = { NOTE_E7, NOTE_E7, 0, NOTE_E7, 
                   0, NOTE_C7, NOTE_E7, 0,
                   NOTE_G7, 0, 0,  0,
                   NOTE_G6, 0, 0, 0, 

                   NOTE_C7, 0, 0, NOTE_G6, 
                   0, 0, NOTE_E6, 0, 
                   0, NOTE_A6, 0, NOTE_B6, 
                   0, NOTE_AS6, NOTE_A6, 0, 

                   NOTE_G6, NOTE_E7, NOTE_G7, 
                   NOTE_A7, 0, NOTE_F7, NOTE_G7, 
                   0, NOTE_E7, 0,NOTE_C7, 
                   NOTE_D7, NOTE_B6, 0, 0,

                   NOTE_C7, 0, 0, NOTE_G6, 
                   0, 0, NOTE_E6, 0, 
                   0, NOTE_A6, 0, NOTE_B6, 
                   0, NOTE_AS6, NOTE_A6, 0, 

                   NOTE_G6, NOTE_E7, NOTE_G7, 
                   NOTE_A7, 0, NOTE_F7, NOTE_G7, 
                   NOTE_D7, NOTE_B6, 0, 0
                 };

  //Mario main them tempo
  int noteDurations[] = { 12, 12, 12, 12, 
                          12, 12, 12, 12,
                          12, 12, 12, 12,
                          12, 12, 12, 12, 

                          12, 12, 12, 12,
                          12, 12, 12, 12, 
                          12, 12, 12, 12, 
                          12, 12, 12, 12, 

                          9, 9, 9,
                          12, 12, 12, 12,
                          12, 12, 12, 12,
                          12, 12, 12, 12,

                          12, 12, 12, 12,
                          12, 12, 12, 12,
                          12, 12, 12, 12,
                          12, 12, 12, 12,

                          9, 9, 9,
                          12, 12, 12, 12,
                          12, 12, 12, 12,
                          12, 12, 12, 12,
                        };

  delay(500);
  
  while ( !digitalRead(ACT_SW) )    // stop music mode with activate switch
  { 
    // iterate over the notes of the melody:
    for (int thisNote = 0; thisNote < 78; thisNote++) 
    {
      if ( !digitalRead(ACT_SW) )   // stop melody with activate stwitch
      {
        int noteDuration = 1000 / noteDurations[thisNote];   // to calculate the note duration, take one second divided by the note type.
                                                             // e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
        tone(BUZZER, melody[thisNote], noteDuration);

        int pauseBetweenNotes = noteDuration * 1.30;    // to distinguish the notes, set a minimum time between them.
                                                        // the note's duration + 30% seems to work well:
        delay(pauseBetweenNotes);
        noTone(BUZZER);   // stop the tone playing:
      }
      else
      {
        break;
      }
    }
  } 

  lcd.setCursor(2,1);
  lcd.print("Stop          ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);
}

void serialmonitor()
{
 int sensorValue;
 
 Serial.begin(9600);    // zet seriele communicatie op 9600 bps

 delay(500);
   
 while( !digitalRead(ACT_SW) )    // stop serial monitor mode with activate switch
 {
  Serial.print("\t");
  Serial.print("Mode sw = ");
  Serial.print(digitalRead(MODE_SW));
  
  Serial.print("\t");
  Serial.print("Line (left) = ");
  Serial.print(digitalRead(LINE_SNR_LFT));  

  Serial.print("\t\t"); 
  Serial.print("Line (right) = ");
  Serial.print(digitalRead(LINE_SNR_RGHT));
   
  sensorValue = analogRead(LGHT_SNR_LFT);  
  Serial.print("\t"); 
  Serial.print("Light (left) = " );
  Serial.print(sensorValue);

  sensorValue = analogRead(LGHT_SNR_RGHT);
  Serial.print("\t"); 
  Serial.print("Light (right) = ");
  Serial.print(sensorValue);

  digitalWrite(DIST_ACT_LFT, HIGH);  
  sensorValue = analogRead(DIST_SNR_LFT);
  Serial.print("\t");
  Serial.print("Distance (left) = ");
  Serial.print(sensorValue);
  digitalWrite(DIST_ACT_LFT, LOW); 

  digitalWrite(DIST_ACT_RGHT, HIGH); 
  sensorValue = analogRead(DIST_SNR_RGHT);
  Serial.print("\t"); 
  Serial.print("Distance (right) = ");
  Serial.print(sensorValue);
  digitalWrite(DIST_ACT_RGHT, LOW); 

  Serial.println(); 
  delay(200); 
 }

  lcd.setCursor(2,1);
  lcd.print("Stop          ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500); 
}

void DC_motor_1()
{
  bool modeSwitch;
  byte motor = 0;

  delay(500);
  
  while ( !digitalRead(ACT_SW) )    // stop DC-motor mode with activate switch
  {
    modeSwitch = digitalRead(MODE_SW);
    if ( modeSwitch )   // modeswitch active 
    {
      motor++; 
      if (motor == 5)
      {
        motor = 0 ;
      }
        
      switch (motor)
      {
         case 1:     // left motor full speed backwards, right motor stop
         lcd.setCursor(0,1);   
         lcd.print("Left backwards ");
         DC_Motor_Backwards(255, 0);
         break;
         
         case 2:    // left motor full speed forwards, right motor stop
         lcd.setCursor(0,1);
         lcd.print("Left forwards  ");
         DC_Motor_Forwards(255, 0);
         break;

         case 3:     // right motor full speed backwards, left motor stop
         lcd.setCursor(0,1);
         lcd.print("Right backwards");
         DC_Motor_Backwards(0, 255);
         break;         

         case 4:     // right motor full speed forwards, left motor stop
         lcd.setCursor(0,1);
         lcd.print("Right forwards ");
         DC_Motor_Forwards(0, 255);
         break;

         default:     // stop both wheels
         lcd.setCursor(0,1);
         lcd.print("Motor Stop     ");
         DC_Motor_Stop(); 
         break;
      } 
    }
    delay(300);   
  }

  DC_Motor_Stop();    // stop both wheels 
  lcd.setCursor(0,1);
  lcd.print(mode);
  lcd.print(":Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);
}

void DC_motor_2()
{
  int i = 0;
  byte trail = 0;
  byte route[] = { 1, 1, 1, 1, 3, 3, 3, 1, 1, 8, 8, 8, 8, 1, 1, 4, 4, 4, 1, 1, 7, 7, 7  } ;  // collection of successive routeparts
  
  delay(500);

  while ( !digitalRead(ACT_SW) )    // stop DC-motor-2 mode with activate switch
  {
    trail = route[i];

    i++;
    if (i == sizeof(route)-1)
    {
      i = 0;
    }
         
    switch (trail)
    {
       case 1:     // straight forwards
       lcd.setCursor(0,1);   
       lcd.print("straight forward");
       DC_Motor_Forwards(150, 150);
       break;

       case 2:     // straight backwards
       lcd.setCursor(0,1);   
       lcd.print("straight backwards");
       DC_Motor_Backwards(150, 150);
       break;       
         
       case 3:    // turn left forwards
       lcd.setCursor(0,1);
       lcd.print("Turn left       ");
       DC_Motor_Forwards(150, 255);
       break;

       case 4:     // turn right forwards
       lcd.setCursor(0,1);
       lcd.print("Turn right     ");
       DC_Motor_Forwards(255, 150);
       break;        

       case 5:    // turn left backwards
       lcd.setCursor(0,1);
       lcd.print("Turn left       ");
       DC_Motor_Backwards(150, 255);
       break;

       case 6:     // turn right backwards
       lcd.setCursor(0,1);
       lcd.print("Turn right     ");
       DC_Motor_Backwards(255, 150);
       break;        

       case 7:     // spinning left
       lcd.setCursor(0,1);
       DC_Motor_Spinning_Right(0, 255);
       break;

       case 8:     // spinning right
       lcd.setCursor(0,1);
       lcd.print("Spinning        ");
       DC_Motor_Spinning_Right(255, 0);
       break;

       default:     // stop both wheels
       lcd.setCursor(0,1);
       lcd.print("Motor Stop     ");
       DC_Motor_Stop(); 
       break;
    }
    delay(500);
  }

  DC_Motor_Stop();    // stop both wheels
  lcd.setCursor(0,1);
  lcd.print(mode);
  lcd.print(":Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);
}
  
void servo()
{
  delay(500);
    
  while ( !digitalRead(ACT_SW) )    // stop servo mode with activate switch 
  {
    for( int i = 1; i < 180; i++)   // servo 1 (pin 15) from 0 to 180 degrees
    {
      if ( !digitalRead(ACT_SW) )   // stop servo cycle with activate switch
      {
        servo_1.write(i);
        delay(50);
      }
      else
      {
        break;
      }
    }

    for( int i = 1; i < 180; i++)   // servo 2 (pin 23) from 0 to 180 degrees
    {
      if ( !digitalRead(ACT_SW) )   // stop servo cycle with activate switch
      {
        servo_1.write(i);
        delay(50);
      }
      else
      {
        break;
      }
    }
    
    for(int i = 180; i > 0; i--)    // servo 1 (pin 15) from 180 to 0 degrees
    {
      if ( !digitalRead(ACT_SW) )   // stop servo cycle with activate switch
      {
        servo_1.write(i);
        delay(50);
      }
      else
      {
        break;
      }
    }

     for(int i = 180; i > 0; i--)    // servo 2 (pin 23) from 180 to 0 degrees
    {
      if ( !digitalRead(ACT_SW) )   // stop servo cycle with activate switch
      {
        servo_1.write(i);
        delay(50);
      }
      else
      {
        break;
      }
    }
  }

  lcd.setCursor(2,1);
  lcd.print("Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);  
}

void lightfollower()
{
  int sensorValue_left;
  int sensorValue_right;

  delay(500);

  while ( !digitalRead(ACT_SW) )    // stop lightfollower mode with activate switch
  {
    sensorValue_left = analogRead(LGHT_SNR_LFT);
    sensorValue_right = analogRead(LGHT_SNR_RGHT); 
  
    if ( ( sensorValue_left >= sensorValue_right - 50 ) && ( sensorValue_left <= sensorValue_right + 50 ) )
    {
      // stop both wheels
      DC_Motor_Stop();    
    }
    else
    {
      if ( sensorValue_left <  sensorValue_right - 50 )
      {
        // left motor forwards, right motor backwards (same speed)
        DC_Motor_Spinning_Right(150, 150); 
      }
      else
      {    
        // left motor backwards, right motor forwards (same speed)
        DC_Motor_Spinning_Left(150, 150);  
      }
    }
    delay(50);
  }

  DC_Motor_Stop();    // stop both wheels
  lcd.setCursor(2,1);
  lcd.print("Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);  
}

void distance()
{
  int sensorValue_left;
  int sensorValue_right;
  
  delay(500);

  while ( !digitalRead(ACT_SW) )    // stop lightfollower mode with activate switch
  {
    digitalWrite(DIST_ACT_LFT, HIGH);   // left IR transmitter on
    delay(2);
    sensorValue_left = analogRead(DIST_SNR_LFT) ;  // read left IR sensor
    digitalWrite(DIST_ACT_LFT, LOW);     // left IR transmitter of
    
    digitalWrite( DIST_ACT_RGHT, HIGH );    // right IR transmitter on
    delay(2);
    sensorValue_right = analogRead(DIST_SNR_RGHT);  // read left IR sensor
    digitalWrite(DIST_ACT_RGHT, LOW );    // right IR transmitter o
    
    if ( sensorValue_left > 500 )   // obstacle on left frontside of car?
    {
      digitalWrite(DIR_MTR_LFT, HIGH);    // yes: left motor backwards
      analogWrite(SPD_MTR_LFT , 50);
    }
    else
    {
      digitalWrite(DIR_MTR_LFT, LOW );    // no: left motor forwards
      analogWrite(SPD_MTR_LFT, 150);
    }
    
    if ( sensorValue_right > 500 )    // obstacle on right frontside of car?
    {
      digitalWrite(DIR_MTR_RGHT, HIGH );  // yes: right motor backwards
      analogWrite(SPD_MTR_RGHT , 100);    
    }
    else
    {
      digitalWrite(DIR_MTR_RGHT, LOW );   // no: right motor forwards
      analogWrite(SPD_MTR_RGHT, 150);
    }

    if ( ( ( sensorValue_left > 500 ) || ( sensorValue_right > 500 ) ) && !digitalRead(ACT_SW) )
    {
      delay(500);
    }
    else
    {
      delay(20);
    }
  }

  DC_Motor_Stop();    // stop both wheels    
  lcd.setCursor(2,1);
  lcd.print("Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);  
}

void linefollower()   
{
  int sensorValue_left;
  int sensorValue_right;

  delay(500);
  
  while ( !digitalRead(ACT_SW) )    // stop linefollower mode with activate switch
  {
    sensorValue_left = digitalRead(4) ;
    sensorValue_right = digitalRead(12) ;

    // both sensors on track
    if ( ( sensorValue_left == HIGH ) && ( sensorValue_right == HIGH ) )
    {
        // both motors same speed
        DC_Motor_Forwards(150, 150);
    }

    // left sensor on track, right sensor off track
    if ( ( sensorValue_left == HIGH ) && ( sensorValue_right == LOW ) )
    {
        // increas speed of right motor
        DC_Motor_Forwards(100, 255);
    }
    
    // left sensor off track, right sensor on track
    if ( ( sensorValue_left == LOW ) && ( sensorValue_right == HIGH ) )
    {
        // increas speed of left motor
        DC_Motor_Forwards(255, 100);
    }
        // both sensors off track
    if ( ( sensorValue_left == LOW ) && ( sensorValue_right == LOW ) )
    {
      DC_Motor_Stop();   // stop both wheels
    }
  }

  DC_Motor_Stop();   // stop both wheels
  lcd.setCursor(2,1);
  lcd.print("Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);  
}

void bluetooth()
{
  delay(500);

  volatile byte startcomms = 0;

  byte dir_Lft_Mtr = 0;
  byte dir_Rght_mtr = 0;

  int spd_Lft_Mtr = 0;
  int spd_Rght_Mtr = 0;

  UBRR1H = (BAUD_PRESCALE >> 8);              // Setting baud rate t0 9600 bps
  UBRR1L = BAUD_PRESCALE;
  UCSR1C = (1 << UCSZ10 ) | (1 << UCSZ11 );   // no parity, 1 stop bit and 8 databits
  UCSR1B = (1 << RXEN1) | (1 << TXEN1) | (1 << RXCIE1);   // TX, RX and RX omplete interrupt enabled
  
//  sei(); //Enable global interrupts

  while ( !digitalRead(ACT_SW) )    // stop bluetooth mode with activate switch 
  {
    if (startcomms == 1)
    {
      if ( (ReceiveY >= 128) && (ReceiveX >= 128) )
      {
        dir_Lft_Mtr = LOW;    // left motor forwards
        dir_Rght_mtr = LOW;   // right motor forwards 
        
        spd_Lft_Mtr = (ReceiveY - 127) * 2;
        if ( spd_Lft_Mtr > 254 ) { spd_Lft_Mtr = 255; }
        if ( spd_Lft_Mtr < 0 ) { spd_Lft_Mtr = 0; }
        
        spd_Rght_Mtr = ( (ReceiveY - 127) * 2) - (ReceiveX - 127);
        if ( spd_Rght_Mtr > 254 ) { spd_Rght_Mtr = 255; }
        if ( spd_Rght_Mtr < 0 ) { spd_Rght_Mtr = 0; }
      }

      else if ( (ReceiveY >= 128) && (ReceiveX <= 127) )
      {
        dir_Lft_Mtr = LOW;    // left motor forwards
        dir_Rght_mtr = LOW;   // right motor forwards
     
        spd_Lft_Mtr = (ReceiveY - 127) * 2;
        if ( spd_Lft_Mtr > 254 ) { spd_Lft_Mtr = 255; }
        if ( spd_Lft_Mtr < 0 ) { spd_Lft_Mtr = 0; }
    
        spd_Rght_Mtr = ( (ReceiveY - 127) * 2 ) - (ReceiveX - 127);
        if ( spd_Rght_Mtr > 254 ) { spd_Rght_Mtr = 255; }
        if ( spd_Rght_Mtr < 0 ) { spd_Rght_Mtr = 0; }
      }
      
      else if ( (ReceiveY <= 127) && (ReceiveX <= 127) )
      {
        dir_Lft_Mtr = HIGH;    // left motor backwards
        dir_Rght_mtr = HIGH;   // right motor backwards 
        
        spd_Lft_Mtr = 255-( ( (127 - ReceiveY) * 2) - (127 - ReceiveX) );
        if (spd_Lft_Mtr > 254) { spd_Lft_Mtr = 255; }
        if (spd_Lft_Mtr < 0) { spd_Lft_Mtr = 0; }
        
        spd_Rght_Mtr = 255-((127-ReceiveY)*2);
        if (spd_Rght_Mtr > 254) { spd_Rght_Mtr = 255; }
        if (spd_Rght_Mtr < 0) { spd_Rght_Mtr = 0; }
      }

      else if ( (ReceiveY <= 127) && (ReceiveX >= 128) ) // forward left
      {
        dir_Lft_Mtr = HIGH;    // left motor backwards
        dir_Rght_mtr = HIGH;   // right motor backwards 
        
        spd_Lft_Mtr = 255 - ( (127-ReceiveY) * 2 );
        if (spd_Lft_Mtr > 254) { spd_Lft_Mtr = 255; }
        if (spd_Lft_Mtr < 0) { spd_Lft_Mtr = 0; }
        
        spd_Rght_Mtr = 255 - ( ( (127-ReceiveY) * 2) - (ReceiveX-127) );
        if (spd_Rght_Mtr > 254) { spd_Rght_Mtr = 255; }
        if (spd_Rght_Mtr < 0) { spd_Rght_Mtr = 0; }
      }

      else
      {
        dir_Lft_Mtr = LOW;   // left motor forwards
        dir_Rght_mtr = LOW;  // right motor forwards 
        spd_Lft_Mtr = 85;
        spd_Rght_Mtr = 85;
      } 

      digitalWrite(DIR_MTR_LFT, dir_Lft_Mtr);
      analogWrite(SPD_MTR_LFT, spd_Lft_Mtr);
      digitalWrite(DIR_MTR_RGHT, dir_Rght_mtr); 
      analogWrite(SPD_MTR_RGHT, spd_Rght_Mtr);
    } 
  }
  
  UCSR1B &= ~(1 << RXCIE1);   // disable RX interrupt
//  cli();    // disable global interrupts
  
  DC_Motor_Stop();   // stop both wheels
  lcd.setCursor(0,1);
  lcd.print(mode);
  lcd.print(":Stop           ");
  while ( digitalRead(ACT_SW) ) {}
  delay(500);
}

ISR (USART1_RX_vect)    // interrupt routine that executes every time a new byte is received
{
    startcomms = 1;
    state = !state;             // inverse the variable "state" to flash led D13
    
    UCSR1B &= ~(1 << RXCIE1);   // disable RX interrupt
    
    ReceiveX = UDR1;    // Fetch the received byte value into the variable " ByteReceived " - the RXint flaq is automatically cleared now
    while ((UCSR1A & (1 << RXC1)) == 0) {};   // wait until byte 2 is received 
    ReceiveY = UDR1;    // receive byte 2
    
    UCSR1B |= (1 << RXCIE1);        //Re-enable RX interrupt  
}

